@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Beraud <adrien.beraud@savoirfairelinux.com>
  *          Alexandre Lision <alexandre.lision@savoirfairelinux.com>
@@ -19,38 +19,39 @@
  */
 package cx.ring.client;
 
-import android.app.AlertDialog;
-import android.app.FragmentManager;
-import android.app.FragmentTransaction;
-import android.content.DialogInterface;
 import android.content.Intent;
 import android.content.res.Configuration;
-import android.graphics.Bitmap;
 import android.graphics.Color;
+import android.graphics.drawable.ColorDrawable;
 import android.os.Bundle;
-import android.support.design.widget.FloatingActionButton;
-import android.support.design.widget.NavigationView;
-import android.support.v4.view.GravityCompat;
-import android.support.v4.widget.DrawerLayout;
-import android.support.v7.app.ActionBar;
-import android.support.v7.app.ActionBarDrawerToggle;
-import android.support.v7.app.AppCompatActivity;
-import android.support.v7.widget.Toolbar;
+
+import com.google.android.material.appbar.AppBarLayout;
+import com.google.android.material.appbar.CollapsingToolbarLayout;
+import com.google.android.material.dialog.MaterialAlertDialogBuilder;
+import com.google.android.material.floatingactionbutton.FloatingActionButton;
+import com.google.android.material.navigation.NavigationView;
+
+import androidx.annotation.NonNull;
+import androidx.coordinatorlayout.widget.CoordinatorLayout;
+import androidx.core.view.GravityCompat;
+import androidx.drawerlayout.widget.DrawerLayout;
+import androidx.appcompat.app.ActionBar;
+import androidx.appcompat.app.ActionBarDrawerToggle;
+import androidx.appcompat.app.AppCompatActivity;
+import androidx.appcompat.widget.Toolbar;
 
 import android.text.TextUtils;
 import android.util.Log;
 import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
-import android.widget.LinearLayout;
-import android.widget.RelativeLayout;
-import android.widget.TextView;
-
-import java.io.File;
-import java.util.List;
 
 import javax.inject.Inject;
 
+import androidx.fragment.app.Fragment;
+import androidx.fragment.app.FragmentManager;
+import androidx.fragment.app.FragmentTransaction;
+
 import butterknife.BindView;
 import butterknife.ButterKnife;
 import cx.ring.BuildConfig;
@@ -62,6 +63,7 @@ import cx.ring.contactrequests.ContactRequestsFragment;
 import cx.ring.fragments.AccountsManagementFragment;
 import cx.ring.fragments.ConversationFragment;
 import cx.ring.fragments.SmartListFragment;
+import cx.ring.interfaces.Colorable;
 import cx.ring.model.Account;
 import cx.ring.model.AccountConfig;
 import cx.ring.navigation.RingNavigationFragment;
@@ -72,19 +74,18 @@ import cx.ring.services.HardwareService;
 import cx.ring.services.NotificationService;
 import cx.ring.services.PreferencesService;
 import cx.ring.settings.SettingsFragment;
-import cx.ring.share.ShareFragment;
-import cx.ring.utils.AndroidFileUtils;
+import cx.ring.settings.VideoSettingsFragment;
 import cx.ring.utils.DeviceUtils;
 import io.reactivex.android.schedulers.AndroidSchedulers;
 import io.reactivex.disposables.CompositeDisposable;
 
-public class HomeActivity extends AppCompatActivity implements RingNavigationFragment.OnNavigationSectionSelected {
+public class HomeActivity extends AppCompatActivity implements RingNavigationFragment.OnNavigationSectionSelected, Colorable {
 
-    public static final int REQUEST_CODE_CREATE_ACCOUNT = 7;
     public static final int REQUEST_CODE_CALL = 3;
     public static final int REQUEST_CODE_CONVERSATION = 4;
     public static final int REQUEST_CODE_PHOTO = 5;
     public static final int REQUEST_CODE_GALLERY = 6;
+    public static final int REQUEST_CODE_QR_CONVERSATION = 7;
     public static final int REQUEST_PERMISSION_CAMERA = 113;
     public static final int REQUEST_PERMISSION_READ_STORAGE = 114;
     public static final String HOME_TAG = "Home";
@@ -92,13 +93,14 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
     public static final String ACCOUNTS_TAG = "Accounts";
     public static final String ABOUT_TAG = "About";
     public static final String SETTINGS_TAG = "Prefs";
-    public static final String SHARE_TAG = "Share";
+    public static final String VIDEO_SETTINGS_TAG = "VideoPrefs";
     static public final String ACTION_PRESENT_TRUST_REQUEST_FRAGMENT = BuildConfig.APPLICATION_ID + "presentTrustRequestFragment";
     static final String TAG = HomeActivity.class.getSimpleName();
     private static final String NAVIGATION_TAG = "Navigation";
-    protected android.app.Fragment fContent;
+    protected Fragment fContent;
     protected RingNavigationFragment fNavigation;
     protected ConversationFragment fConversation;
+
     @Inject
     DeviceRuntimeService mDeviceRuntimeService;
 
@@ -116,26 +118,32 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
     NavigationView mNavigationView;
     @BindView(R.id.drawer_layout)
     DrawerLayout mNavigationDrawer;
+
     @BindView(R.id.main_toolbar)
     Toolbar mToolbar;
-    @BindView(R.id.toolbar_spacer)
-    LinearLayout mToolbarSpacerView;
-    @BindView(R.id.toolbar_spacer_title)
-    TextView mToolbarSpacerTitle;
+    @BindView(R.id.app_bar)
+    AppBarLayout mAppBar;
+    @BindView(R.id.toolbar_layout)
+    CollapsingToolbarLayout mToolbarLayout;
+
     @BindView(R.id.action_button)
     FloatingActionButton actionButton;
+
     @BindView(R.id.content_frame)
-    RelativeLayout mFrameLayout;
+    ViewGroup mFrameLayout;
+
     private boolean mIsMigrationDialogAlreadyShowed;
     private ActionBarDrawerToggle mDrawerToggle;
     private Boolean isDrawerLocked = false;
     private String mAccountWithPendingrequests = null;
     private float mToolbarSize;
+    private float mToolbarElevation;
+
     private final CompositeDisposable mDisposable = new CompositeDisposable();
 
     /* called before activity is killed, e.g. rotation */
     @Override
-    protected void onSaveInstanceState(Bundle bundle) {
+    protected void onSaveInstanceState(@NonNull Bundle bundle) {
         super.onSaveInstanceState(bundle);
     }
 
@@ -143,11 +151,12 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
         RingApplication.getInstance().startDaemon();
-
         mToolbarSize = getResources().getDimension(R.dimen.abc_action_bar_default_height_material);
+        mToolbarElevation = getResources().getDimension(R.dimen.toolbar_elevation);
+        FragmentManager fragmentManager = getSupportFragmentManager();
 
         if (savedInstanceState != null) {
-            fNavigation = (RingNavigationFragment) getFragmentManager().findFragmentByTag(NAVIGATION_TAG);
+            fNavigation = (RingNavigationFragment) fragmentManager.findFragmentByTag(NAVIGATION_TAG);
         }
         setContentView(R.layout.activity_home);
 
@@ -195,7 +204,7 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
 
         if (fNavigation == null && savedInstanceState == null) {
             fNavigation = new RingNavigationFragment();
-            getFragmentManager().beginTransaction()
+            fragmentManager.beginTransaction()
                     .replace(R.id.navigation_container, fNavigation, NAVIGATION_TAG)
                     .commit();
         }
@@ -213,9 +222,7 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
             handleShareIntent(intent);
         }
 
-        setVideoEnabledFromPermission();
 
-        FragmentManager fragmentManager = getFragmentManager();
         fContent = fragmentManager.findFragmentById(R.id.main_frame);
         if (fNavigation != null) {
             onNavigationViewReady();
@@ -234,6 +241,12 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
 
     }
 
+    @Override
+    protected void onDestroy() {
+        super.onDestroy();
+        fContent = null;
+    }
+
     private void handleShareIntent(Intent intent) {
         String action = intent.getAction();
         if (Intent.ACTION_SEND.equals(action) || Intent.ACTION_SEND_MULTIPLE.equals(action)) {
@@ -269,7 +282,7 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
             return;
         }
 
-        if (!getFragmentManager().findFragmentByTag(HOME_TAG).isVisible()) {
+        if (!getSupportFragmentManager().findFragmentByTag(HOME_TAG).isVisible()) {
             fNavigation.selectSection(RingNavigationFragment.Section.HOME);
             onNavigationSectionSelected(RingNavigationFragment.Section.HOME);
         }
@@ -281,25 +294,16 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
     }
 
     private void showMigrationDialog() {
-
         if (mIsMigrationDialogAlreadyShowed) {
             return;
         }
-
         mIsMigrationDialogAlreadyShowed = true;
-
-        new AlertDialog.Builder(HomeActivity.this)
+        new MaterialAlertDialogBuilder(this)
                 .setTitle(R.string.account_migration_title_dialog)
                 .setMessage(R.string.account_migration_message_dialog)
-                .setIcon(R.drawable.ic_warning)
-                .setCancelable(true)
-                .setPositiveButton(android.R.string.ok, (dialog, which) -> {
-                    dialog.dismiss();
-                    fNavigation.selectSection(RingNavigationFragment.Section.MANAGE);
-                })
-                .setNegativeButton(android.R.string.cancel, (dialog, which) -> dialog.dismiss())
-                .setOnCancelListener(DialogInterface::dismiss)
-                .setOnDismissListener(DialogInterface::dismiss)
+                .setIcon(R.drawable.baseline_warning_24)
+                .setPositiveButton(android.R.string.ok, (dialog, which) -> fNavigation.selectSection(RingNavigationFragment.Section.MANAGE))
+                .setNegativeButton(android.R.string.cancel, null)
                 .show();
     }
 
@@ -310,45 +314,37 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
     }
 
     @Override
-    public void onConfigurationChanged(Configuration newConfig) {
+    public void onConfigurationChanged(@NonNull Configuration newConfig) {
         super.onConfigurationChanged(newConfig);
         mDrawerToggle.onConfigurationChanged(newConfig);
     }
 
-    @Override
-    protected void onStart() {
-        String path = AndroidFileUtils.ringtonesPath(this);
-        if (!(new File(path + "/default.wav")).exists()) {
-            Log.d(TAG, "onStart: default.wav doesn't exist. Copying ringtones.");
-            AndroidFileUtils.copyAssetFolder(getAssets(), "ringtones", path);
+    public void setToolbarTop(boolean top) {
+        if (top) {
+            mAppBar.setElevation(0);
+        } else {
+            mAppBar.setElevation(mToolbarElevation);
         }
-
-        super.onStart();
     }
 
     public void setToolbarState(boolean doubleHeight, int titleRes) {
-
-        mToolbar.setMinimumHeight((int) mToolbarSize);
-        ViewGroup.LayoutParams toolbarSpacerViewParams = mToolbarSpacerView.getLayoutParams();
-
+        ActionBar ab = getSupportActionBar();
+        if (ab != null)
+            ab.setIcon(null);
+        mToolbar.setLogo(null);
         if (doubleHeight) {
-            // setting the height of the toolbar spacer with the same height than the toolbar
-            toolbarSpacerViewParams.height = (int) mToolbarSize;
-            mToolbarSpacerView.setLayoutParams(toolbarSpacerViewParams);
-
-            // setting the toolbar spacer title (hiding the real toolbar title)
-            mToolbarSpacerTitle.setText(titleRes);
-            mToolbar.setTitle("");
-
-            // the spacer and the action button become visible
-            mToolbarSpacerView.setVisibility(View.VISIBLE);
-            actionButton.show();
+            mAppBar.setElevation(mToolbarElevation);
+            mToolbarLayout.setTitleEnabled(true);
+            mToolbarLayout.setTitle(getString(titleRes));
         } else {
-            // hide the toolbar spacer and the action button
-            mToolbarSpacerView.setVisibility(View.GONE);
-            actionButton.hide();
-            mToolbar.setTitle(titleRes);
+            mAppBar.setElevation(0);
+            mToolbarLayout.setTitleEnabled(false);
+            mToolbar.setTitle(getString(titleRes));
         }
+        CoordinatorLayout.LayoutParams params = (CoordinatorLayout.LayoutParams) mAppBar.getLayoutParams();
+        params.height = doubleHeight ? (int) (2.01 * mToolbarSize) : CoordinatorLayout.LayoutParams.WRAP_CONTENT;
+        mAppBar.setLayoutParams(params);
+        mAppBar.setExpanded(doubleHeight);
     }
 
     public FloatingActionButton getActionButton() {
@@ -362,22 +358,21 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
         mDisposable.clear();
         mDisposable.add(mAccountService.getObservableAccountList()
                 .observeOn(AndroidSchedulers.mainThread())
-                .subscribe(accounts ->  {
-                        for (Account account : accounts) {
-                            if (account.needsMigration()) {
-                                showMigrationDialog();
-                                break;
-                            }
+                .subscribe(accounts -> {
+                    for (Account account : accounts) {
+                        if (account.needsMigration()) {
+                            showMigrationDialog();
+                            break;
                         }
-                    }));
-        setVideoEnabledFromPermission();
+                    }
+                }));
     }
 
     public void startConversationTablet(Bundle bundle) {
         fConversation = new ConversationFragment();
         fConversation.setArguments(bundle);
 
-        getFragmentManager().beginTransaction()
+        getSupportFragmentManager().beginTransaction()
                 .replace(R.id.conversation_container, fConversation, ConversationFragment.class.getName())
                 .commit();
     }
@@ -393,7 +388,7 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
         fContent = new ContactRequestsFragment();
         fContent.setArguments(bundle);
         fNavigation.selectSection(RingNavigationFragment.Section.CONTACT_REQUESTS);
-        getFragmentManager().beginTransaction()
+        getSupportFragmentManager().beginTransaction()
                 .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
                 .replace(R.id.main_frame, fContent, CONTACT_REQUESTS_TAG)
                 .addToBackStack(CONTACT_REQUESTS_TAG).commit();
@@ -411,59 +406,40 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
             mNavigationDrawer.closeDrawer(GravityCompat.START);
             return;
         }
-        if (getFragmentManager().getBackStackEntryCount() > 1) {
-            popCustomBackStack();
-            fNavigation.selectSection(RingNavigationFragment.Section.HOME);
+        FragmentManager fragmentManager = getSupportFragmentManager();
+        int fCount = fragmentManager.getBackStackEntryCount();
+        if (fCount > 1) {
+            FragmentManager.BackStackEntry entry = fragmentManager.getBackStackEntryAt(fCount - 2);
+            fContent = fragmentManager.findFragmentById(entry.getId());
+            fragmentManager.popBackStack();
+            if (fCount == 2)
+                fNavigation.selectSection(RingNavigationFragment.Section.HOME);
             return;
         }
-
         finish();
     }
 
     private void popCustomBackStack() {
-        FragmentManager fragmentManager = getFragmentManager();
+        FragmentManager fragmentManager = getSupportFragmentManager();
         FragmentManager.BackStackEntry entry = fragmentManager.getBackStackEntryAt(0);
-        fContent = fragmentManager.findFragmentByTag(entry.getName());
-        for (int i = 0; i < fragmentManager.getBackStackEntryCount() - 1; ++i) {
+        fContent = fragmentManager.findFragmentById(entry.getId());
+        int entryCount = fragmentManager.getBackStackEntryCount() - 1;
+        for (int i = 0; i < entryCount; ++i) {
             fragmentManager.popBackStack();
         }
     }
 
-    // TODO: Remove this when low level services are ready
     public void onNavigationViewReady() {
         if (fNavigation != null) {
-            fNavigation.setNavigationSectionSelectedListener(HomeActivity.this);
+            fNavigation.setNavigationSectionSelectedListener(this);
         }
     }
 
     @Override
-    public boolean onOptionsItemSelected(MenuItem item) {
+    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
         return mDrawerToggle.onOptionsItemSelected(item);
     }
 
-    @Override
-    public void onActivityResult(int requestCode, int resultCode, Intent data) {
-        super.onActivityResult(requestCode, resultCode, data);
-
-        switch (requestCode) {
-            case REQUEST_CODE_CALL:
-                if (resultCode == CallActivity.RESULT_FAILURE) {
-                    Log.w(TAG, "Call Failed");
-                }
-                break;
-            case REQUEST_CODE_PHOTO:
-                if (resultCode == RESULT_OK && data != null) {
-                    fNavigation.updatePhoto((Bitmap) data.getExtras().get("data"));
-                }
-                break;
-            case REQUEST_CODE_GALLERY:
-                if (resultCode == RESULT_OK && data != null) {
-                    fNavigation.updatePhoto(data.getData());
-                }
-                break;
-        }
-    }
-
     @Override
     public void onNavigationSectionSelected(RingNavigationFragment.Section section) {
         if (!isDrawerLocked) {
@@ -475,12 +451,13 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
                 if (fContent instanceof SmartListFragment) {
                     break;
                 }
-                if (getFragmentManager().getBackStackEntryCount() == 1) {
+                FragmentManager manager = getSupportFragmentManager();
+                if (manager.getBackStackEntryCount() == 1) {
                     break;
                 }
 
                 popCustomBackStack();
-                fContent = getFragmentManager().findFragmentByTag(HOME_TAG);
+                fContent = manager.findFragmentByTag(HOME_TAG);
                 break;
             case CONTACT_REQUESTS:
                 Bundle bundle = new Bundle();
@@ -489,9 +466,10 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
                     ((ContactRequestsFragment) fContent).presentForAccount(bundle);
                     break;
                 }
+                popCustomBackStack();
                 fContent = new ContactRequestsFragment();
                 fContent.setArguments(bundle);
-                getFragmentManager().beginTransaction()
+                getSupportFragmentManager().beginTransaction()
                         .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
                         .replace(R.id.main_frame, fContent, CONTACT_REQUESTS_TAG)
                         .addToBackStack(CONTACT_REQUESTS_TAG).commit();
@@ -500,8 +478,9 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
                 if (fContent instanceof AccountsManagementFragment) {
                     break;
                 }
+                popCustomBackStack();
                 fContent = new AccountsManagementFragment();
-                getFragmentManager().beginTransaction()
+                getSupportFragmentManager().beginTransaction()
                         .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
                         .replace(R.id.main_frame, fContent, ACCOUNTS_TAG)
                         .addToBackStack(ACCOUNTS_TAG).commit();
@@ -510,8 +489,9 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
                 if (fContent instanceof AboutFragment) {
                     break;
                 }
+                popCustomBackStack();
                 fContent = new AboutFragment();
-                getFragmentManager().beginTransaction()
+                getSupportFragmentManager().beginTransaction()
                         .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
                         .replace(R.id.main_frame, fContent, ABOUT_TAG)
                         .addToBackStack(ABOUT_TAG).commit();
@@ -519,9 +499,6 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
             case SETTINGS:
                 this.goToSettings();
                 break;
-            case SHARE:
-                goToShare();
-                break;
             default:
                 break;
         }
@@ -553,17 +530,6 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
         startActivityForResult(intent, AccountsManagementFragment.ACCOUNT_CREATE_REQUEST);
     }
 
-    private void goToShare() {
-        if (fContent instanceof ShareFragment) {
-            return;
-        }
-        fContent = new ShareFragment();
-
-        getFragmentManager().beginTransaction()
-                .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
-                .replace(R.id.main_frame, fContent, SHARE_TAG)
-                .addToBackStack(SHARE_TAG).commit();
-    }
 
     public void goToSettings() {
         if (mNavigationDrawer != null && !isDrawerLocked) {
@@ -572,21 +538,33 @@ public class HomeActivity extends AppCompatActivity implements RingNavigationFra
         if (fContent instanceof SettingsFragment) {
             return;
         }
+        popCustomBackStack();
         fContent = new SettingsFragment();
-        getFragmentManager()
+        getSupportFragmentManager()
                 .beginTransaction()
                 .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
                 .replace(R.id.main_frame, fContent, SETTINGS_TAG)
                 .addToBackStack(SETTINGS_TAG).commit();
     }
 
-    private void setVideoEnabledFromPermission() {
-        //~ Setting correct VIDEO_ENABLED value based on the state of the
-        //~ permission. It can handle the case where the user decides to remove a permission from
-        //~ the Android general settings.
-        if (!mDeviceRuntimeService.hasVideoPermission()) {
-            mAccountService.setAccountsVideoEnabled(false);
+    public void goToVideoSettings() {
+        if (mNavigationDrawer != null && !isDrawerLocked) {
+            mNavigationDrawer.closeDrawers();
         }
+        if (fContent instanceof VideoSettingsFragment) {
+            return;
+        }
+        fContent = new VideoSettingsFragment();
+        getSupportFragmentManager()
+                .beginTransaction()
+                .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
+                .replace(R.id.main_frame, fContent, VIDEO_SETTINGS_TAG)
+                .addToBackStack(VIDEO_SETTINGS_TAG).commit();
+    }
+
+    @Override
+    public void setColor(int color) {
+        //mToolbar.setBackground(new ColorDrawable(color));
     }
 
     public interface Refreshable {
