@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Thibault Wittemberg <thibault.wittemberg@savoirfairelinux.com>
  *  Author: Adrien Beraud <adrien.beraud@savoirfairelinux.com>
@@ -22,7 +22,11 @@ package cx.ring.services;
 
 import android.content.Context;
 import android.content.SharedPreferences;
-import android.support.annotation.NonNull;
+import androidx.annotation.NonNull;
+import androidx.appcompat.app.AppCompatDelegate;
+import androidx.preference.PreferenceManager;
+
+import android.os.Build;
 import android.text.TextUtils;
 
 import java.util.HashMap;
@@ -32,19 +36,27 @@ import java.util.Set;
 
 import javax.inject.Inject;
 
+import cx.ring.R;
 import cx.ring.application.RingApplication;
 import cx.ring.model.Settings;
+import cx.ring.utils.DeviceUtils;
 import cx.ring.utils.NetworkUtils;
 
 public class SharedPreferencesServiceImpl extends PreferencesService {
 
-    private static final String RING_SETTINGS = "ring_settings";
+    public static final String RING_SETTINGS = "ring_settings";
     private static final String RING_REQUESTS = "ring_requests";
-    private static final String RING_MOBILE_DATA = "mobile_data";
+    public static final String PREFS_THEME = "theme";
+    public static final String PREFS_VIDEO = "videoPrefs";
     private static final String RING_PUSH_NOTIFICATIONS = "push_notifs";
+    private static final String RING_PERSISTENT_NOTIFICATION = "persistent_notif";
+    private static final String RING_HW_ENCODING = "video_hwenc";
+    public static final String RING_BITRATE = "video_bitrate";
+    public static final String RING_RESOLUTION = "video_resolution";
     private static final String RING_SYSTEM_CONTACTS = "system_contacts";
     private static final String RING_PLACE_CALLS = "place_calls";
     private static final String RING_ON_STARTUP = "on_startup";
+    public static final String PREF_DARK_MODE= "darkMode";
     private final Map<String, Set<String>> mNotifiedRequests = new HashMap<>();
 
     @Inject
@@ -52,29 +64,29 @@ public class SharedPreferencesServiceImpl extends PreferencesService {
 
     @Override
     protected void saveSettings(Settings settings) {
-        SharedPreferences appPrefs = mContext.getSharedPreferences(RING_SETTINGS, Context.MODE_PRIVATE);
+        SharedPreferences appPrefs = getPreferences();
         SharedPreferences.Editor edit = appPrefs.edit();
         edit.clear();
-        edit.putBoolean(RING_MOBILE_DATA, settings.isAllowMobileData());
         edit.putBoolean(RING_SYSTEM_CONTACTS, settings.isAllowSystemContacts());
         edit.putBoolean(RING_PLACE_CALLS, settings.isAllowPlaceSystemCalls());
         edit.putBoolean(RING_ON_STARTUP, settings.isAllowRingOnStartup());
         edit.putBoolean(RING_PUSH_NOTIFICATIONS, settings.isAllowPushNotifications());
+        edit.putBoolean(RING_PERSISTENT_NOTIFICATION, settings.isAllowPersistentNotification());
         edit.apply();
     }
 
     @Override
     protected Settings loadSettings() {
-        SharedPreferences appPrefs = mContext.getSharedPreferences(RING_SETTINGS, Context.MODE_PRIVATE);
+        SharedPreferences appPrefs = getPreferences();
         Settings settings = getUserSettings();
         if (settings == null) {
             settings = new Settings();
         }
-        settings.setAllowMobileData(appPrefs.getBoolean(RING_MOBILE_DATA, false));
         settings.setAllowSystemContacts(appPrefs.getBoolean(RING_SYSTEM_CONTACTS, false));
         settings.setAllowPlaceSystemCalls(appPrefs.getBoolean(RING_PLACE_CALLS, false));
         settings.setAllowRingOnStartup(appPrefs.getBoolean(RING_ON_STARTUP, true));
         settings.setAllowPushNotifications(appPrefs.getBoolean(RING_PUSH_NOTIFICATIONS, false));
+        settings.setAllowPersistentNotification(appPrefs.getBoolean(RING_PERSISTENT_NOTIFICATION, false));
         return settings;
     }
 
@@ -113,7 +125,7 @@ public class SharedPreferencesServiceImpl extends PreferencesService {
 
     @Override
     public boolean hasNetworkConnected() {
-        return NetworkUtils.isConnectivityAllowed(mContext, getSettings().isAllowMobileData());
+        return NetworkUtils.isConnectivityAllowed(mContext);
     }
 
     @Override
@@ -121,4 +133,61 @@ public class SharedPreferencesServiceImpl extends PreferencesService {
         String token = RingApplication.getInstance().getPushToken();
         return getSettings().isAllowPushNotifications() && !TextUtils.isEmpty(token) /*&& NetworkUtils.isPushAllowed(mContext, getSettings().isAllowMobileData())*/;
     }
+
+    @Override
+    public int getResolution() {
+        return Integer.parseInt(getVideoPreferences().getString(RING_RESOLUTION,
+                DeviceUtils.isTv(mContext)
+                        ? mContext.getString(R.string.video_resolution_default_tv)
+                        : mContext.getString(R.string.video_resolution_default)));
+    }
+
+    @Override
+    public int getBitrate() {
+        return Integer.parseInt(getVideoPreferences().getString(RING_BITRATE, mContext.getString(R.string.video_bitrate_default)));
+    }
+
+    @Override
+    public boolean isHardwareAccelerationEnabled() {
+        return getVideoPreferences().getBoolean(RING_HW_ENCODING, true);
+    }
+
+    @Override
+    public void setDarkMode(boolean enabled) {
+        SharedPreferences.Editor edit = getThemePreferences().edit();
+        edit.putBoolean(PREF_DARK_MODE, enabled)
+                .apply();
+        applyDarkMode(enabled);
+    }
+
+    @Override
+    public boolean getDarkMode() {
+        return getThemePreferences().getBoolean(PREF_DARK_MODE, false);
+    }
+
+    @Override
+    public void loadDarkMode() {
+        applyDarkMode(getDarkMode());
+    }
+
+    private void applyDarkMode(boolean enabled) {
+        AppCompatDelegate.setDefaultNightMode(
+                enabled ? AppCompatDelegate.MODE_NIGHT_YES
+                        : Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q
+                            ? AppCompatDelegate.MODE_NIGHT_FOLLOW_SYSTEM
+                            : AppCompatDelegate.MODE_NIGHT_AUTO_BATTERY);
+    }
+
+    private SharedPreferences getPreferences() {
+        return mContext.getSharedPreferences(RING_SETTINGS, Context.MODE_PRIVATE);
+    }
+
+    private SharedPreferences getVideoPreferences() {
+        return mContext.getSharedPreferences(PREFS_VIDEO, Context.MODE_PRIVATE);
+    }
+
+    private SharedPreferences getThemePreferences() {
+        return PreferenceManager.getDefaultSharedPreferences(mContext);
+    }
+
 }
