@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
  *
  *  Author: Thibault Wittemberg <thibault.wittemberg@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
@@ -19,8 +19,8 @@
  */
 package cx.ring.services;
 
+import java.util.Map;
 import java.util.concurrent.ScheduledExecutorService;
-import java.util.concurrent.TimeUnit;
 
 import javax.inject.Inject;
 import javax.inject.Named;
@@ -33,7 +33,6 @@ import cx.ring.daemon.IntVect;
 import cx.ring.daemon.IntegerMap;
 import cx.ring.daemon.PresenceCallback;
 import cx.ring.daemon.Ringservice;
-import cx.ring.daemon.RingserviceJNI;
 import cx.ring.daemon.StringMap;
 import cx.ring.daemon.StringVect;
 import cx.ring.daemon.UintVect;
@@ -44,8 +43,6 @@ public class DaemonService {
 
     private static final String TAG = DaemonService.class.getSimpleName();
 
-    private static final int POLLING_TIMEOUT = 50;
-
     @Inject
     @Named("DaemonExecutor")
     ScheduledExecutorService mExecutor;
@@ -62,9 +59,6 @@ public class DaemonService {
     @Inject
     protected HardwareService mHardwareService;
 
-    @Inject
-    protected PresenceService mPresenceService;
-
     @Inject
     protected AccountService mAccountService;
 
@@ -95,7 +89,7 @@ public class DaemonService {
         return mDaemonStarted;
     }
 
-    public void startDaemon() {
+    synchronized public void startDaemon() {
         if (!mDaemonStarted) {
             mDaemonStarted = true;
             Log.i(TAG, "Starting daemon ...");
@@ -105,18 +99,12 @@ public class DaemonService {
             mConfigurationCallback = new DaemonConfigurationCallback();
             mDataCallback = new DaemonDataTransferCallback();
             Ringservice.init(mConfigurationCallback, mCallAndConferenceCallback, mPresenceCallback, mDataCallback, mHardwareCallback);
-            startRingServicePolling();
             Log.i(TAG, "DaemonService started");
         }
     }
 
-    private void startRingServicePolling() {
-        mExecutor.scheduleAtFixedRate(RingserviceJNI::pollEvents, 0, POLLING_TIMEOUT, TimeUnit.MILLISECONDS);
-    }
-
-    public void stopDaemon() {
+    synchronized public void stopDaemon() {
         mExecutor.shutdown();
-
         if (mDaemonStarted) {
             Log.i(TAG, "stopping daemon ...");
             Ringservice.fini();
@@ -134,7 +122,7 @@ public class DaemonService {
 
         @Override
         public void accountsChanged() {
-            mAccountService.accountsChanged();
+            mExecutor.submit(() -> mAccountService.accountsChanged());
         }
 
         @Override
@@ -144,27 +132,37 @@ public class DaemonService {
 
         @Override
         public void registrationStateChanged(String accountId, String newState, int code, String detailString) {
-            mAccountService.registrationStateChanged(accountId, newState, code, detailString);
+            mExecutor.submit(() -> mAccountService.registrationStateChanged(accountId, newState, code, detailString));
         }
 
         @Override
         public void volatileAccountDetailsChanged(String account_id, StringMap details) {
-            mAccountService.volatileAccountDetailsChanged(account_id, details);
+            Map<String, String> jdetails = details.toNative();
+            mExecutor.submit(() -> mAccountService.volatileAccountDetailsChanged(account_id, jdetails));
+        }
+
+        @Override
+        public void accountDetailsChanged(String account_id, StringMap details) {
+            Map<String, String> jdetails = details.toNative();
+            mExecutor.submit(() -> mAccountService.accountDetailsChanged(account_id, jdetails));
         }
 
         @Override
         public void incomingAccountMessage(String accountId, String from, StringMap messages) {
-            mAccountService.incomingAccountMessage(accountId, null, from, messages);
+            if (messages == null || messages.empty())
+                return;
+            Map<String, String> jmessages = messages.toNativeFromUtf8();
+            mExecutor.submit(() -> mAccountService.incomingAccountMessage(accountId, null, from, jmessages));
         }
 
         @Override
         public void accountMessageStatusChanged(String accountId, long messageId, String to, int status) {
-            mAccountService.accountMessageStatusChanged(accountId, messageId, to, status);
+            mExecutor.submit(() -> mAccountService.accountMessageStatusChanged(accountId, messageId, to, status));
         }
 
         @Override
         public void errorAlert(int alert) {
-            mAccountService.errorAlert(alert);
+            mExecutor.submit(() -> mAccountService.errorAlert(alert));
         }
 
         @Override
@@ -184,7 +182,8 @@ public class DaemonService {
 
         @Override
         public void knownDevicesChanged(String accountId, StringMap devices) {
-            mAccountService.knownDevicesChanged(accountId, devices);
+            Map<String, String> jdevices = devices.toNativeFromUtf8();
+            mExecutor.submit(() -> mAccountService.knownDevicesChanged(accountId, jdevices));
         }
 
         @Override
@@ -214,17 +213,18 @@ public class DaemonService {
 
         @Override
         public void incomingTrustRequest(String accountId, String from, Blob message, long received) {
-            mAccountService.incomingTrustRequest(accountId, from, message.toJavaString(), received);
+            String jmessage = message.toJavaString();
+            mExecutor.submit(() -> mAccountService.incomingTrustRequest(accountId, from, jmessage, received));
         }
 
         @Override
         public void contactAdded(String accountId, String uri, boolean confirmed) {
-            mAccountService.contactAdded(accountId, uri, confirmed);
+            mExecutor.submit(() -> mAccountService.contactAdded(accountId, uri, confirmed));
         }
 
         @Override
         public void contactRemoved(String accountId, String uri, boolean banned) {
-            mAccountService.contactRemoved(accountId, uri, banned);
+            mExecutor.submit(() -> mAccountService.contactRemoved(accountId, uri, banned));
         }
     }
 
@@ -232,17 +232,20 @@ public class DaemonService {
 
         @Override
         public void callStateChanged(String callId, String newState, int detailCode) {
-            mCallService.callStateChanged(callId, newState, detailCode);
+            mExecutor.submit(() -> mCallService.callStateChanged(callId, newState, detailCode));
         }
 
         @Override
         public void incomingCall(String accountId, String callId, String from) {
-            mCallService.incomingCall(accountId, callId, from);
+            mExecutor.submit(() -> mCallService.incomingCall(accountId, callId, from));
         }
 
         @Override
         public void incomingMessage(String callId, String from, StringMap messages) {
-            mCallService.incomingMessage(callId, from, messages);
+            if (messages == null || messages.empty())
+                return;
+            Map<String, String> jmessages = messages.toNativeFromUtf8();
+            mExecutor.submit(() -> mCallService.incomingMessage(callId, from, jmessages));
         }
 
         @Override
@@ -276,22 +279,22 @@ public class DaemonService {
 
         @Override
         public void newServerSubscriptionRequest(String remote) {
-            mPresenceService.newServerSubscriptionRequest(remote);
+            Log.d(TAG, "newServerSubscriptionRequest: " + remote);
         }
 
         @Override
         public void serverError(String accountId, String error, String message) {
-            mPresenceService.serverError(accountId, error, message);
+            Log.d(TAG, "serverError: " + accountId + ", " + error + ", " + message);
         }
 
         @Override
         public void newBuddyNotification(String accountId, String buddyUri, int status, String lineStatus) {
-            mPresenceService.newBuddyNotification(accountId, buddyUri, status, lineStatus);
+            mAccountService.getAccount(accountId).presenceUpdate(buddyUri, status == 1);
         }
 
         @Override
         public void subscriptionStateChanged(String accountId, String buddyUri, int state) {
-            mPresenceService.subscriptionStateChanged(accountId, buddyUri, state);
+            Log.d(TAG, "subscriptionStateChanged: " + accountId + ", " + buddyUri + ", " + state);
         }
     }
 
@@ -317,6 +320,11 @@ public class DaemonService {
             mHardwareService.setParameters(camId, format, width, height, rate);
         }
 
+        @Override
+        public void requestKeyFrame() {
+            mHardwareService.requestKeyFrame();
+        }
+
         @Override
         public void startCapture(String camId) {
             Log.d(TAG, "startCapture: " + camId);
