@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2015-2019 Savoir-faire Linux Inc.
  *  Author: Stepan Salenikovich <stepan.salenikovich@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,19 @@ GdkPixbuf *ring_frame_avatar(GdkPixbuf *avatar);
 
 GdkPixbuf *ring_draw_unread_messages(const GdkPixbuf *avatar, int unread_count);
 
-GdkPixbuf *ring_draw_presence(const GdkPixbuf *avatar, bool present);
+gboolean   draw_qrcode(cairo_t* cr, const std::string& to_encode, uint32_t size);
+
+
+enum class IconStatus {
+    ABSENT,
+    PRESENT,
+    DISCONNECTED,
+    TRYING,
+    CONNECTED,
+    INVALID
+};
+GdkPixbuf *ring_draw_status(const GdkPixbuf *avatar, IconStatus status);
+
+GdkRGBA get_ambient_color(GtkWidget* widget);
 
 #endif /* _DRAWING */
