@@ -1088,7 +1088,7 @@ IceTransport::packIceMsg(uint8_t version) const
 }
 
 bool
-IceTransport::getCandidateFromSDP(const std::string& line, IceCandidate& cand)
+IceTransport::getCandidateFromSDP(const std::string& line, IceCandidate& cand) const
 {
     /**   Section 4.5, RFC 6544 (https://tools.ietf.org/html/rfc6544)
      *    candidate-attribute   = "candidate" ":" foundation SP component-id SP
@@ -1244,7 +1244,7 @@ IceTransport::send(int comp_id, const unsigned char* buf, size_t len)
           pimpl_->waitDataCv_.wait(lk);
           current_size = pimpl_->lastReadLen_[comp_id];
         }
-    } else if (status != PJ_SUCCESS) {
+    } else if (status != PJ_SUCCESS && status != PJ_EPENDING) {
         if (status == PJ_EBUSY) {
             errno = EAGAIN;
         } else {
