@@ -3,6 +3,7 @@
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
+ *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -51,7 +52,7 @@
 #include <cstdlib>
 #include <unistd.h>
 
-namespace ring { namespace tls {
+namespace jami { namespace tls {
 
 static constexpr const char* DTLS_CERT_PRIORITY_STRING {"SECURE192:-VERS-TLS-ALL:+VERS-DTLS-ALL:-RSA:%SERVER_PRECEDENCE:%SAFE_RENEGOTIATION"};
 static constexpr const char* DTLS_FULL_PRIORITY_STRING {"SECURE192:-KX-ALL:+ANON-ECDH:+ANON-DH:+SECURE192:-VERS-TLS-ALL:+VERS-DTLS-ALL:-RSA:%SERVER_PRECEDENCE:%SAFE_RENEGOTIATION"};
@@ -100,7 +101,7 @@ public:
     TlsCertificateCredendials() {
         int ret = gnutls_certificate_allocate_credentials(&creds_);
         if (ret < 0) {
-            RING_ERR("gnutls_certificate_allocate_credentials() failed with ret=%d", ret);
+            JAMI_ERR("gnutls_certificate_allocate_credentials() failed with ret=%d", ret);
             throw std::bad_alloc();
         }
     }
@@ -123,7 +124,7 @@ public:
     TlsAnonymousClientCredendials() {
         int ret = gnutls_anon_allocate_client_credentials(&creds_);
         if (ret < 0) {
-            RING_ERR("gnutls_anon_allocate_client_credentials() failed with ret=%d", ret);
+            JAMI_ERR("gnutls_anon_allocate_client_credentials() failed with ret=%d", ret);
             throw std::bad_alloc();
         }
     }
@@ -146,7 +147,7 @@ public:
     TlsAnonymousServerCredendials() {
         int ret = gnutls_anon_allocate_server_credentials(&creds_);
         if (ret < 0) {
-            RING_ERR("gnutls_anon_allocate_server_credentials() failed with ret=%d", ret);
+            JAMI_ERR("gnutls_anon_allocate_server_credentials() failed with ret=%d", ret);
             throw std::bad_alloc();
         }
     }
@@ -187,6 +188,10 @@ public:
 
     SocketType& transport_;
 
+    // State protectors
+    std::mutex stateMutex_;
+    std::condition_variable stateCondition_;
+
     // State machine
     TlsSessionState handleStateSetup(TlsSessionState state);
     TlsSessionState handleStateCookie(TlsSessionState state);
@@ -196,6 +201,7 @@ public:
     TlsSessionState handleStateShutdown(TlsSessionState state);
     std::map<TlsSessionState, StateHandler> fsmHandlers_ {};
     std::atomic<TlsSessionState> state_ {TlsSessionState::SETUP};
+    std::atomic<TlsSessionState> newState_ {TlsSessionState::NONE};
     std::atomic<int> maxPayload_ {-1};
 
     // IO GnuTLS <-> ICE
@@ -216,7 +222,7 @@ public:
     ssize_t sendRaw(const void*, size_t);
     ssize_t sendRawVec(const giovec_t*, int);
     ssize_t recvRaw(void*, size_t);
-    int waitForRawData(unsigned);
+    int waitForRawData(std::chrono::milliseconds);
 
     bool initFromRecordState(int offset=0);
     void handleDataPacket(std::vector<ValueType>&&, uint64_t);
@@ -233,6 +239,7 @@ public:
     std::unique_ptr<TlsAnonymousClientCredendials> cacred_; // ctor init.
     std::unique_ptr<TlsAnonymousServerCredendials> sacred_; // ctor init.
     std::unique_ptr<TlsCertificateCredendials> xcred_; // ctor init.
+    std::mutex sessionMutex_;
     gnutls_session_t session_ {nullptr};
     gnutls_datum_t cookie_key_ {nullptr, 0};
     gnutls_dtls_prestate_st prestate_ {};
@@ -249,9 +256,6 @@ public:
     bool setup();
     void process();
     void cleanup();
-    // State protectors
-    std::mutex stateMutex_;
-    std::condition_variable stateCondition_;
 
     ScheduledExecutor scheduler_;
 
@@ -300,6 +304,9 @@ TlsSession::TlsSessionImpl::TlsSessionImpl(SocketType& transport,
 
 TlsSession::TlsSessionImpl::~TlsSessionImpl()
 {
+    state_ = TlsSessionState::SHUTDOWN;
+    stateCondition_.notify_all();
+    rxCv_.notify_all();
     thread_.join();
     if (not transport_.isReliable())
         transport_.setOnRecv(nullptr);
@@ -314,7 +321,7 @@ TlsSession::TlsSessionImpl::typeName() const
 void
 TlsSession::TlsSessionImpl::dump_io_stats() const
 {
-    RING_DBG("[TLS] RxRawPkt=%zu (%zu bytes) - TxRawPkt=%zu (%zu bytes)",
+    JAMI_DBG("[TLS] RxRawPkt=%zu (%zu bytes) - TxRawPkt=%zu (%zu bytes)",
              stRxRawPacketCnt_.load(), stRxRawBytesCnt_.load(),
              stTxRawPacketCnt_.load(), stTxRawBytesCnt_.load());
 }
@@ -327,14 +334,14 @@ TlsSession::TlsSessionImpl::setupClient()
     if (not transport_.isReliable()) {
         ret = gnutls_init(&session_, GNUTLS_CLIENT | GNUTLS_DATAGRAM);
         // uncoment to reactivate PMTUD
-        // RING_DBG("[TLS] set heartbeat reception for retrocompatibility check on server");
+        // JAMI_DBG("[TLS] set heartbeat reception for retrocompatibility check on server");
         // gnutls_heartbeat_enable(session_,GNUTLS_HB_PEER_ALLOWED_TO_SEND);
     } else {
         ret = gnutls_init(&session_, GNUTLS_CLIENT);
     }
 
     if (ret != GNUTLS_E_SUCCESS) {
-        RING_ERR("[TLS] session init failed: %s", gnutls_strerror(ret));
+        JAMI_ERR("[TLS] session init failed: %s", gnutls_strerror(ret));
         return TlsSessionState::SHUTDOWN;
     }
 
@@ -354,7 +361,7 @@ TlsSession::TlsSessionImpl::setupServer()
         ret = gnutls_init(&session_, GNUTLS_SERVER | GNUTLS_DATAGRAM);
 
         // uncoment to reactivate PMTUD
-        // RING_DBG("[TLS] set heartbeat reception");
+        // JAMI_DBG("[TLS] set heartbeat reception");
         // gnutls_heartbeat_enable(session_, GNUTLS_HB_PEER_ALLOWED_TO_SEND);
 
         gnutls_dtls_prestate_set(session_, &prestate_);
@@ -363,7 +370,7 @@ TlsSession::TlsSessionImpl::setupServer()
     }
 
     if (ret != GNUTLS_E_SUCCESS) {
-        RING_ERR("[TLS] session init failed: %s", gnutls_strerror(ret));
+        JAMI_ERR("[TLS] session init failed: %s", gnutls_strerror(ret));
         return TlsSessionState::SHUTDOWN;
     }
 
@@ -389,7 +396,7 @@ TlsSession::TlsSessionImpl::initAnonymous()
         if (const auto& dh_params = params_.dh_params.get().get())
             gnutls_anon_set_server_dh_params(*sacred_, dh_params);
         else
-            RING_WARN("[TLS] DH params unavailable"); // YOMGUI: need to stop?
+            JAMI_WARN("[TLS] DH params unavailable"); // YOMGUI: need to stop?
     }
 }
 
@@ -421,14 +428,14 @@ TlsSession::TlsSessionImpl::initCredentials()
             throw std::runtime_error("can't load CA " + params_.ca_list + ": "
                                      + std::string(gnutls_strerror(ret)));
 
-        RING_DBG("[TLS] CA list %s loadev", params_.ca_list.c_str());
+        JAMI_DBG("[TLS] CA list %s loadev", params_.ca_list.c_str());
     }
     if (params_.peer_ca) {
         auto chain = params_.peer_ca->getChainWithRevocations();
         auto ret = gnutls_certificate_set_x509_trust(*xcred_, chain.first.data(), chain.first.size());
         if (not chain.second.empty())
             gnutls_certificate_set_x509_crl(*xcred_, chain.second.data(), chain.second.size());
-        RING_DBG("[TLS] Peer CA list %lu (%lu CRLs): %d", chain.first.size(), chain.second.size(), ret);
+        JAMI_DBG("[TLS] Peer CA list %lu (%lu CRLs): %d", chain.first.size(), chain.second.size(), ret);
     }
 
     // Load user-given identity (key and passwd)
@@ -446,7 +453,7 @@ TlsSession::TlsSessionImpl::initCredentials()
             throw std::runtime_error("can't load certificate: "
                                      + std::string(gnutls_strerror(ret)));
 
-        RING_DBG("[TLS] User identity loaded");
+        JAMI_DBG("[TLS] User identity loaded");
     }
 
     // Setup DH-params (server only, may block on dh_params.get())
@@ -454,7 +461,7 @@ TlsSession::TlsSessionImpl::initCredentials()
         if (const auto& dh_params = params_.dh_params.get().get())
             gnutls_certificate_set_dh_params(*xcred_, dh_params);
         else
-            RING_WARN("[TLS] DH params unavailable"); // YOMGUI: need to stop?
+            JAMI_WARN("[TLS] DH params unavailable"); // YOMGUI: need to stop?
     }
 }
 
@@ -469,7 +476,7 @@ TlsSession::TlsSessionImpl::commonSessionInit()
                                          transport_.isReliable() ? TLS_FULL_PRIORITY_STRING : DTLS_FULL_PRIORITY_STRING,
                                          nullptr);
         if (ret != GNUTLS_E_SUCCESS) {
-            RING_ERR("[TLS] TLS priority set failed: %s", gnutls_strerror(ret));
+            JAMI_ERR("[TLS] TLS priority set failed: %s", gnutls_strerror(ret));
             return false;
         }
 
@@ -480,7 +487,7 @@ TlsSession::TlsSessionImpl::commonSessionInit()
             ret = gnutls_credentials_set(session_, GNUTLS_CRD_ANON, *cacred_);
 
         if (ret != GNUTLS_E_SUCCESS) {
-            RING_ERR("[TLS] anonymous credential set failed: %s", gnutls_strerror(ret));
+            JAMI_ERR("[TLS] anonymous credential set failed: %s", gnutls_strerror(ret));
             return false;
         }
     } else {
@@ -489,7 +496,7 @@ TlsSession::TlsSessionImpl::commonSessionInit()
                                          transport_.isReliable() ? TLS_CERT_PRIORITY_STRING : DTLS_CERT_PRIORITY_STRING,
                                          nullptr);
         if (ret != GNUTLS_E_SUCCESS) {
-            RING_ERR("[TLS] TLS priority set failed: %s", gnutls_strerror(ret));
+            JAMI_ERR("[TLS] TLS priority set failed: %s", gnutls_strerror(ret));
             return false;
         }
     }
@@ -497,7 +504,7 @@ TlsSession::TlsSessionImpl::commonSessionInit()
     // Add certificate credentials
     ret = gnutls_credentials_set(session_, GNUTLS_CRD_CERTIFICATE, *xcred_);
     if (ret != GNUTLS_E_SUCCESS) {
-        RING_ERR("[TLS] certificate credential set failed: %s", gnutls_strerror(ret));
+        JAMI_ERR("[TLS] certificate credential set failed: %s", gnutls_strerror(ret));
         return false;
     }
     gnutls_certificate_send_x509_rdn_sequence(session_, 0);
@@ -529,7 +536,7 @@ TlsSession::TlsSessionImpl::commonSessionInit()
     gnutls_transport_set_pull_timeout_function(session_,
                                                [](gnutls_transport_ptr_t t, unsigned ms) -> int {
                                                    auto this_ = reinterpret_cast<TlsSessionImpl*>(t);
-                                                   return this_->waitForRawData(ms);
+                                                   return this_->waitForRawData(std::chrono::milliseconds(ms));
                                                });
 
     return true;
@@ -564,7 +571,7 @@ TlsSession::TlsSessionImpl::send(const ValueType* tx_data, std::size_t tx_size,
              * state has not changed, so we have to ask for more data first.
              * We will just try again later, although this should never happen.
              */
-            RING_ERR() << "[TLS] send failed (only " << total_written << " bytes sent): "
+            JAMI_ERR() << "[TLS] send failed (only " << total_written << " bytes sent): "
                        << gnutls_strerror(nwritten);
             ec = std::error_code(nwritten, std::system_category());
             return 0;
@@ -594,10 +601,10 @@ TlsSession::TlsSessionImpl::sendRaw(const void* buf, size_t size)
         }
 
         if (ec.value() == EAGAIN) {
-            RING_WARN() << "[TLS] EAGAIN from transport, retry#" << ++retry_count;
+            JAMI_WARN() << "[TLS] EAGAIN from transport, retry#" << ++retry_count;
             std::this_thread::sleep_for(std::chrono::milliseconds(10));
             if (retry_count == 100) {
-                RING_ERR() << "[TLS] excessive retry detected, aborting";
+                JAMI_ERR() << "[TLS] excessive retry detected, aborting";
                 ec.assign(EIO, std::system_category());
             }
         }
@@ -605,7 +612,7 @@ TlsSession::TlsSessionImpl::sendRaw(const void* buf, size_t size)
 
     // Must be called to pass errno value to GnuTLS on Windows (cf. GnuTLS doc)
     gnutls_transport_set_errno(session_, ec.value());
-    RING_ERR() << "[TLS] transport failure on tx: errno = " << ec.value();
+    JAMI_ERR() << "[TLS] transport failure on tx: errno = " << ec.value();
     return -1;
 }
 
@@ -658,16 +665,21 @@ TlsSession::TlsSessionImpl::recvRaw(void* buf, size_t size)
 // 'timeout' is in milliseconds.
 // Should return 0 on timeout, a positive number if data are available for read, or -1 on error.
 int
-TlsSession::TlsSessionImpl::waitForRawData(unsigned timeout)
+TlsSession::TlsSessionImpl::waitForRawData(std::chrono::milliseconds timeout)
 {
     if (transport_.isReliable()) {
         std::error_code ec;
-        if (transport_.waitForData(timeout, ec) <= 0) {
+        auto err = transport_.waitForData(timeout, ec);
+        if (err <= 0) {
             // shutdown?
             if (state_ == TlsSessionState::SHUTDOWN) {
                 gnutls_transport_set_errno(session_, EINTR);
                 return -1;
             }
+            if (ec) {
+                gnutls_transport_set_errno(session_, ec.value());
+                return -1;
+            }
             return 0;
         }
         return 1;
@@ -675,13 +687,13 @@ TlsSession::TlsSessionImpl::waitForRawData(unsigned timeout)
 
     // non-reliable uses callback installed with setOnRecv()
     std::unique_lock<std::mutex> lk {rxMutex_};
-    rxCv_.wait_for(lk, std::chrono::milliseconds(timeout), [this]{ return !rxQueue_.empty() or state_ == TlsSessionState::SHUTDOWN; });
+    rxCv_.wait_for(lk, timeout, [this]{ return !rxQueue_.empty() or state_ == TlsSessionState::SHUTDOWN; });
     if (state_ == TlsSessionState::SHUTDOWN) {
         gnutls_transport_set_errno(session_, EINTR);
         return -1;
     }
     if (rxQueue_.empty()) {
-        RING_ERR("[TLS] waitForRawData: timeout after %u ms", timeout);
+        JAMI_ERR("[TLS] waitForRawData: timeout after %ld ms", timeout.count());
         return 0;
     }
     return 1;
@@ -692,14 +704,14 @@ TlsSession::TlsSessionImpl::initFromRecordState(int offset)
 {
     std::array<uint8_t, 8> seq;
     if (gnutls_record_get_state(session_, 1, nullptr, nullptr, nullptr, &seq[0]) != GNUTLS_E_SUCCESS) {
-        RING_ERR("[TLS] Fatal-error Unable to read initial state");
+        JAMI_ERR("[TLS] Fatal-error Unable to read initial state");
         return false;
     }
 
     baseSeq_ = array2uint(seq) + offset;
     gapOffset_ = baseSeq_;
     lastRxSeq_ = baseSeq_ - 1;
-    RING_DBG("[TLS] Initial sequence number: %lx", baseSeq_);
+    JAMI_DBG("[TLS] Initial sequence number: %lx", baseSeq_);
     return true;
 }
 
@@ -723,30 +735,35 @@ TlsSession::TlsSessionImpl::cleanup()
     state_ = TlsSessionState::SHUTDOWN; // be sure to block any user operations
     stateCondition_.notify_all();
 
-    if (session_) {
-        if (transport_.isReliable())
-            gnutls_bye(session_, GNUTLS_SHUT_RDWR);
-        else
-            gnutls_bye(session_, GNUTLS_SHUT_WR); // not wait for a peer answer
-        gnutls_deinit(session_);
-        session_ = nullptr;
+    {
+        std::lock_guard<std::mutex> lk(sessionMutex_);
+        if (session_) {
+            if (transport_.isReliable())
+                gnutls_bye(session_, GNUTLS_SHUT_RDWR);
+            else
+                gnutls_bye(session_, GNUTLS_SHUT_WR); // not wait for a peer answer
+            gnutls_deinit(session_);
+            session_ = nullptr;
+        }
     }
 
     if (cookie_key_.data)
         gnutls_free(cookie_key_.data);
+
+    transport_.shutdown();
 }
 
 TlsSessionState
 TlsSession::TlsSessionImpl::handleStateSetup(UNUSED TlsSessionState state)
 {
-    RING_DBG("[TLS] Start %s session", typeName());
+    JAMI_DBG("[TLS] Start %s session", typeName());
 
     try {
         if (anonymous_)
             initAnonymous();
         initCredentials();
     } catch (const std::exception& e) {
-        RING_ERR("[TLS] authentifications init failed: %s", e.what());
+        JAMI_ERR("[TLS] authentifications init failed: %s", e.what());
         return TlsSessionState::SHUTDOWN;
     }
 
@@ -764,7 +781,7 @@ TlsSession::TlsSessionImpl::handleStateSetup(UNUSED TlsSessionState state)
 TlsSessionState
 TlsSession::TlsSessionImpl::handleStateCookie(TlsSessionState state)
 {
-    RING_DBG("[TLS] SYN cookie");
+    JAMI_DBG("[TLS] SYN cookie");
 
     std::size_t count;
     {
@@ -773,7 +790,7 @@ TlsSession::TlsSessionImpl::handleStateCookie(TlsSessionState state)
         if (!rxCv_.wait_for(lk, COOKIE_TIMEOUT,
                             [this]{ return !rxQueue_.empty()
                                     or state_ == TlsSessionState::SHUTDOWN; })) {
-            RING_ERR("[TLS] SYN cookie failed: timeout");
+            JAMI_ERR("[TLS] SYN cookie failed: timeout");
             return TlsSessionState::SHUTDOWN;
         }
         // Shutdown state?
@@ -815,7 +832,7 @@ TlsSession::TlsSessionImpl::handleStateCookie(TlsSessionState state)
         // So we retry until we get a valid cookie.
         // To protect against a flood attack we delay each retry after FLOOD_THRESHOLD rx bytes.
         if (cookie_count_ >= FLOOD_THRESHOLD) {
-            RING_WARN("[TLS] flood threshold reach (retry in %zds)",
+            JAMI_WARN("[TLS] flood threshold reach (retry in %zds)",
                       std::chrono::duration_cast<std::chrono::seconds>(FLOOD_PAUSE).count());
             dump_io_stats();
             std::this_thread::sleep_for(FLOOD_PAUSE); // flood attack protection
@@ -823,7 +840,7 @@ TlsSession::TlsSessionImpl::handleStateCookie(TlsSessionState state)
         return state;
     }
 
-    RING_DBG("[TLS] cookie ok");
+    JAMI_DBG("[TLS] cookie ok");
 
     return setupServer();
 }
@@ -834,7 +851,7 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
     int ret;
     size_t retry_count = 0;
     do {
-        RING_DBG("[TLS] handshake");
+        JAMI_DBG("[TLS] handshake");
         ret = gnutls_handshake(session_);
     } while ((ret == GNUTLS_E_INTERRUPTED   or
               ret == GNUTLS_E_AGAIN       ) and
@@ -842,7 +859,7 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
 
     // Stop on fatal error
     if (gnutls_error_is_fatal(ret)) {
-        RING_ERR("[TLS] handshake failed: %s", gnutls_strerror(ret));
+        JAMI_ERR("[TLS] handshake failed: %s", gnutls_strerror(ret));
         return TlsSessionState::SHUTDOWN;
     }
 
@@ -850,7 +867,7 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
     if (ret != GNUTLS_E_SUCCESS) {
         // TODO: handle GNUTLS_E_LARGE_PACKET (MTU must be lowered)
         if (ret != GNUTLS_E_AGAIN)
-            RING_DBG("[TLS] non-fatal handshake error: %s", gnutls_strerror(ret));
+            JAMI_DBG("[TLS] non-fatal handshake error: %s", gnutls_strerror(ret));
         return state;
     }
 
@@ -863,7 +880,7 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
     if (!isTLS1_3 || (isTLS1_3 && isServer_)) {
 #endif
         if (!gnutls_safe_renegotiation_status(session_)) {
-            RING_ERR("[TLS] server identity changed! MiM attack?");
+            JAMI_ERR("[TLS] server identity changed! MiM attack?");
             return TlsSessionState::SHUTDOWN;
         }
 #if GNUTLS_VERSION_NUMBER >= 0x030605
@@ -871,20 +888,20 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
 #endif
 
     auto desc = gnutls_session_get_desc(session_);
-    RING_DBG("[TLS] session established: %s", desc);
+    JAMI_DBG("[TLS] session established: %s", desc);
     gnutls_free(desc);
 
     // Anonymous connection? rehandshake immediately with certificate authentification forced
     auto cred = gnutls_auth_get_type(session_);
     if (cred == GNUTLS_CRD_ANON) {
-        RING_DBG("[TLS] renogotiate with certificate authentification");
+        JAMI_DBG("[TLS] renogotiate with certificate authentification");
 
         // Re-setup TLS algorithms priority list with only certificate based cipher suites
         ret = gnutls_priority_set_direct(session_,
                                          transport_.isReliable() ? TLS_CERT_PRIORITY_STRING : DTLS_CERT_PRIORITY_STRING,
                                          nullptr);
         if (ret != GNUTLS_E_SUCCESS) {
-            RING_ERR("[TLS] session TLS cert-only priority set failed: %s", gnutls_strerror(ret));
+            JAMI_ERR("[TLS] session TLS cert-only priority set failed: %s", gnutls_strerror(ret));
             return TlsSessionState::SHUTDOWN;
         }
 
@@ -892,14 +909,14 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
         gnutls_credentials_clear(session_);
         ret = gnutls_credentials_set(session_, GNUTLS_CRD_CERTIFICATE, *xcred_);
         if (ret != GNUTLS_E_SUCCESS) {
-            RING_ERR("[TLS] session credential set failed: %s", gnutls_strerror(ret));
+            JAMI_ERR("[TLS] session credential set failed: %s", gnutls_strerror(ret));
             return TlsSessionState::SHUTDOWN;
         }
 
         return state; // handshake
 
     } else if (cred != GNUTLS_CRD_CERTIFICATE) {
-        RING_ERR("[TLS] spurious session credential (%u)", cred);
+        JAMI_ERR("[TLS] spurious session credential (%u)", cred);
         return TlsSessionState::SHUTDOWN;
     }
 
@@ -926,13 +943,13 @@ TlsSession::TlsSessionImpl::handleStateMtuDiscovery(UNUSED TlsSessionState state
         if (!isServer_) {
             pathMtuHeartbeat();
             if (state_ == TlsSessionState::SHUTDOWN) {
-                RING_ERR("[TLS] session destroyed while performing PMTUD, shuting down");
+                JAMI_ERR("[TLS] session destroyed while performing PMTUD, shuting down");
                 return TlsSessionState::SHUTDOWN;
             }
             pmtudOver_ = true;
         }
     } else {
-        RING_ERR() << "[TLS] PEER HEARTBEAT DISABLED: using transport MTU value " << mtuProbe_;
+        JAMI_ERR() << "[TLS] PEER HEARTBEAT DISABLED: using transport MTU value " << mtuProbe_;
         pmtudOver_ = true;
     }
 
@@ -940,7 +957,7 @@ TlsSession::TlsSessionImpl::handleStateMtuDiscovery(UNUSED TlsSessionState state
     maxPayload_ = gnutls_dtls_get_data_mtu(session_);
 
     if (pmtudOver_) {
-        RING_DBG() << "[TLS] maxPayload: " << maxPayload_.load();
+        JAMI_DBG() << "[TLS] maxPayload: " << maxPayload_.load();
         if (!initFromRecordState())
             return TlsSessionState::SHUTDOWN;
     }
@@ -960,7 +977,7 @@ TlsSession::TlsSessionImpl::handleStateMtuDiscovery(UNUSED TlsSessionState state
 void
 TlsSession::TlsSessionImpl::pathMtuHeartbeat()
 {
-    RING_DBG() << "[TLS] PMTUD: starting probing with " << HEARTBEAT_RETRANS_TIMEOUT.count()
+    JAMI_DBG() << "[TLS] PMTUD: starting probing with " << HEARTBEAT_RETRANS_TIMEOUT.count()
                << "ms of retransmission timeout";
 
     gnutls_heartbeat_set_timeouts(session_,
@@ -975,7 +992,7 @@ TlsSession::TlsSessionImpl::pathMtuHeartbeat()
     // Hence we have to signal to the TLS session to reduce the MTU on client size accordingly.
     if (transport_.localAddr().isIpv4() and transport_.remoteAddr().isIpv6()) {
         mtuOffset = ASYMETRIC_TRANSPORT_MTU_OFFSET;
-        RING_WARN() << "[TLS] local/remote IP protocol version not alike, use an MTU offset of "
+        JAMI_WARN() << "[TLS] local/remote IP protocol version not alike, use an MTU offset of "
                     << ASYMETRIC_TRANSPORT_MTU_OFFSET << " bytes to compensate";
     }
 
@@ -984,7 +1001,7 @@ TlsSession::TlsSessionImpl::pathMtuHeartbeat()
     for (auto mtu: MTUS_) {
         gnutls_dtls_set_mtu(session_, mtu);
         auto data_mtu = gnutls_dtls_get_data_mtu(session_);
-        RING_DBG() << "[TLS] PMTUD: mtu " << mtu
+        JAMI_DBG() << "[TLS] PMTUD: mtu " << mtu
                    << ", payload " << data_mtu;
         auto bytesToSend = data_mtu - mtuOffset - 3; // want to know why -3? ask gnutls!
 
@@ -993,27 +1010,27 @@ TlsSession::TlsSessionImpl::pathMtuHeartbeat()
         } while (errno_send == GNUTLS_E_AGAIN || (errno_send == GNUTLS_E_INTERRUPTED && state_ != TlsSessionState::SHUTDOWN));
 
         if (errno_send != GNUTLS_E_SUCCESS) {
-            RING_DBG() << "[TLS] PMTUD: mtu " << mtu << " [FAILED]";
+            JAMI_DBG() << "[TLS] PMTUD: mtu " << mtu << " [FAILED]";
             break;
         }
 
         mtuProbe_ = mtu;
-        RING_DBG() << "[TLS] PMTUD: mtu " << mtu << " [OK]";
+        JAMI_DBG() << "[TLS] PMTUD: mtu " << mtu << " [OK]";
     }
 
     if (errno_send == GNUTLS_E_TIMEDOUT) { // timeout is considered as a packet loss, then the good mtu is the precedent
         if (mtuProbe_ == MTUS_[0]) {
-            RING_WARN() << "[TLS] PMTUD: no response on first ping, using minimal MTU value "
+            JAMI_WARN() << "[TLS] PMTUD: no response on first ping, using minimal MTU value "
                         << mtuProbe_;
         } else {
-            RING_WARN() << "[TLS] PMTUD: timed out, using last working mtu "
+            JAMI_WARN() << "[TLS] PMTUD: timed out, using last working mtu "
                         << mtuProbe_;
         }
     } else if (errno_send != GNUTLS_E_SUCCESS) {
-        RING_ERR() << "[TLS] PMTUD: failed with gnutls error '"
+        JAMI_ERR() << "[TLS] PMTUD: failed with gnutls error '"
                    << gnutls_strerror(errno_send) << '\'';
     } else {
-        RING_DBG() << "[TLS] PMTUD: reached maximal value";
+        JAMI_DBG() << "[TLS] PMTUD: reached maximal value";
     }
 }
 
@@ -1027,14 +1044,14 @@ TlsSession::TlsSessionImpl::handleDataPacket(std::vector<ValueType>&& buf, uint6
     } else {
         // too old?
         if (seq_delta <= -MISS_ORDERING_LIMIT) {
-            RING_WARN("[TLS] drop old pkt: 0x%lx", pkt_seq);
+            JAMI_WARN("[TLS] drop old pkt: 0x%lx", pkt_seq);
             return;
         }
 
         // No duplicate check as DTLS prevents that for us (replay protection)
 
         // accept Out-Of-Order pkt - will be reordered by queue flush operation
-        RING_WARN("[TLS] OOO pkt: 0x%lx", pkt_seq);
+        JAMI_WARN("[TLS] OOO pkt: 0x%lx", pkt_seq);
     }
 
     {
@@ -1085,9 +1102,9 @@ TlsSession::TlsSessionImpl::flushRxQueue()
     if ((now - lastReadTime_) >= RX_OOO_TIMEOUT) {
         // OOO packet timeout - consider waited packets as lost
         if (auto lost = next_offset - gapOffset_)
-            RING_WARN("[TLS] %lu lost since 0x%lx", lost, gapOffset_);
+            JAMI_WARN("[TLS] %lu lost since 0x%lx", lost, gapOffset_);
         else
-            RING_WARN("[TLS] slow flush");
+            JAMI_WARN("[TLS] slow flush");
     } else if (next_offset != gapOffset_)
         return;
 
@@ -1117,11 +1134,20 @@ TlsSession::TlsSessionImpl::handleStateEstablished(TlsSessionState state)
     // Nothing to do in reliable mode, so just wait for state change
     if (transport_.isReliable()) {
         auto disconnected = [this]() ->  bool {
-            return state_.load() != TlsSessionState::ESTABLISHED;
+            return state_.load() != TlsSessionState::ESTABLISHED
+             or newState_.load() != TlsSessionState::NONE;
         };
         std::unique_lock<std::mutex> lk(stateMutex_);
         stateCondition_.wait(lk, disconnected);
-        return state;
+        auto oldState = state_.load();
+        if (oldState == TlsSessionState::ESTABLISHED) {
+            auto newState = newState_.load();
+            if (newState != TlsSessionState::NONE) {
+                newState_ = TlsSessionState::NONE;
+                return newState;
+            }
+        }
+        return oldState;
     }
 
     // block until rx packet or state change
@@ -1144,7 +1170,7 @@ TlsSession::TlsSessionImpl::handleStateEstablished(TlsSessionState state)
             gnutls_dtls_set_mtu(session_, mtuProbe_);
             maxPayload_ = gnutls_dtls_get_data_mtu(session_);
             pmtudOver_ = true;
-            RING_DBG() << "[TLS] maxPayload: " << maxPayload_.load();
+            JAMI_DBG() << "[TLS] maxPayload: " << maxPayload_.load();
 
             if (!initFromRecordState(-1))
                 return TlsSessionState::SHUTDOWN;
@@ -1154,24 +1180,24 @@ TlsSession::TlsSessionImpl::handleStateEstablished(TlsSessionState state)
         handleDataPacket(std::move(rawPktBuf_), array2uint(seq));
         // no state change
     } else if (ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED) {
-        RING_DBG("[TLS] PMTUD: ping received sending pong");
+        JAMI_DBG("[TLS] PMTUD: ping received sending pong");
         auto errno_send = gnutls_heartbeat_pong(session_, 0);
 
         if (errno_send != GNUTLS_E_SUCCESS){
-            RING_ERR("[TLS] PMTUD: failed on pong with error %d: %s", errno_send,
+            JAMI_ERR("[TLS] PMTUD: failed on pong with error %d: %s", errno_send,
                       gnutls_strerror(errno_send));
         } else {
             ++hbPingRecved_;
         }
         // no state change
     } else if (ret == 0) {
-        RING_DBG("[TLS] eof");
+        JAMI_DBG("[TLS] eof");
         state = TlsSessionState::SHUTDOWN;
     } else if (ret == GNUTLS_E_REHANDSHAKE) {
-        RING_DBG("[TLS] re-handshake");
+        JAMI_DBG("[TLS] re-handshake");
         state = TlsSessionState::HANDSHAKE;
     } else if (gnutls_error_is_fatal(ret)) {
-        RING_ERR("[TLS] fatal error in recv: %s", gnutls_strerror(ret));
+        JAMI_ERR("[TLS] fatal error in recv: %s", gnutls_strerror(ret));
         state = TlsSessionState::SHUTDOWN;
     } // else non-fatal error... let's continue
 
@@ -1181,7 +1207,7 @@ TlsSession::TlsSessionImpl::handleStateEstablished(TlsSessionState state)
 TlsSessionState
 TlsSession::TlsSessionImpl::handleStateShutdown(TlsSessionState state)
 {
-    RING_DBG("[TLS] shutdown");
+    JAMI_DBG("[TLS] shutdown");
 
     // Stop ourself
     thread_.stop();
@@ -1214,9 +1240,7 @@ TlsSession::TlsSession(SocketType& transport, const TlsParams& params,
 {}
 
 TlsSession::~TlsSession()
-{
-    shutdown();
-}
+{}
 
 bool
 TlsSession::isInitiator() const
@@ -1234,8 +1258,8 @@ int
 TlsSession::maxPayload() const
 {
     if (pimpl_->state_ == TlsSessionState::SHUTDOWN)
-        throw std::runtime_error("Getting MTU from non-valid TLS session");
-    return gnutls_dtls_get_data_mtu(pimpl_->session_);
+        throw std::runtime_error("Getting maxPayload from non-valid TLS session");
+    return pimpl_->transport_.maxPayload();
 }
 
 const char*
@@ -1257,7 +1281,7 @@ TlsSession::currentCipherSuiteId(std::array<uint8_t, 2>& cs_id) const
     }
 
     auto name = gnutls_cipher_get_name(s_cipher);
-    RING_WARN("[TLS] No Cipher Suite Id found for cipher %s", name ? name : "<null>");
+    JAMI_WARN("[TLS] No Cipher Suite Id found for cipher %s", name ? name : "<null>");
     return {};
 }
 
@@ -1268,7 +1292,6 @@ TlsSession::shutdown()
     pimpl_->state_ = TlsSessionState::SHUTDOWN;
     pimpl_->stateCondition_.notify_all();
     pimpl_->rxCv_.notify_one(); // unblock waiting FSM
-    pimpl_->transport_.shutdown();
 }
 
 std::size_t
@@ -1293,25 +1316,39 @@ TlsSession::read(ValueType* data, std::size_t size, std::error_code& ec)
     }
 
     while (true) {
-        auto ret = gnutls_record_recv(pimpl_->session_, data, size);
+        ssize_t ret;
+        {
+            std::lock_guard<std::mutex> lk(pimpl_->sessionMutex_);
+            if (!pimpl_->session_) return 0;
+            ret = gnutls_record_recv(pimpl_->session_, data, size);
+        }
         if (ret > 0) {
             ec.clear();
             return ret;
         }
 
+        std::lock_guard<std::mutex> lk(pimpl_->stateMutex_);
         if (ret == 0) {
-            RING_DBG("[TLS] eof");
-            shutdown();
+            if (pimpl_) {
+                JAMI_DBG("[TLS] eof");
+                pimpl_->newState_ = TlsSessionState::SHUTDOWN;
+                pimpl_->stateCondition_.notify_all();
+                pimpl_->rxCv_.notify_one(); // unblock waiting FSM
+            }
             error = std::errc::broken_pipe;
             break;
         } else if (ret == GNUTLS_E_REHANDSHAKE) {
-            RING_DBG("[TLS] re-handshake");
-            pimpl_->state_ = TlsSessionState::HANDSHAKE;
+            JAMI_DBG("[TLS] re-handshake");
+            pimpl_->newState_ = TlsSessionState::HANDSHAKE;
             pimpl_->rxCv_.notify_one(); // unblock waiting FSM
             pimpl_->stateCondition_.notify_all();
         } else if (gnutls_error_is_fatal(ret)) {
-            RING_ERR("[TLS] fatal error in recv: %s", gnutls_strerror(ret));
-            shutdown();
+            if (pimpl_ && pimpl_->state_ != TlsSessionState::SHUTDOWN) {
+                JAMI_ERR("[TLS] fatal error in recv: %s", gnutls_strerror(ret));
+                pimpl_->newState_ = TlsSessionState::SHUTDOWN;
+                pimpl_->stateCondition_.notify_all();
+                pimpl_->rxCv_.notify_one(); // unblock waiting FSM
+            }
             error = std::errc::io_error;
             break;
         }
@@ -1339,11 +1376,11 @@ TlsSession::waitForReady(const std::chrono::steady_clock::duration& timeout)
 }
 
 int
-TlsSession::waitForData(unsigned ms_timeout, std::error_code& ec) const
+TlsSession::waitForData(std::chrono::milliseconds timeout, std::error_code& ec) const
 {
-    if (!pimpl_->transport_.waitForData(ms_timeout, ec))
+    if (!pimpl_->transport_.waitForData(timeout, ec))
         return 0;
     return 1;
 }
 
-}} // namespace ring::tls
+}} // namespace jami::tls
